@@ -10,7 +10,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Id: pgtcl.c,v 1.2 2006-09-10 00:24:39+00 lbayuk Exp $
+ *	  $Id: pgtcl.c 334 2013-10-04 15:04:44Z lbayuk $
  *
  *-------------------------------------------------------------------------
  */
@@ -19,41 +19,8 @@
 #include "pgtclCmds.h"
 #include "pgtclId.h"
 
-/* BEGIN STUBS MUMBO JUMBO http://mini.net/tcl/1687 */
-/* We need at least the Tcl_Obj interface that was started in 8.0 */
-#if TCL_MAJOR_VERSION < 8
-#error "we need Tcl 8.0 or greater to build this"
-
-/* Check for Stubs compatibility when asked for it. */
-#elif defined(USE_TCL_STUBS) && TCL_MAJOR_VERSION == 8 && \
-		(TCL_MINOR_VERSION == 0 || \
-		(TCL_MINOR_VERSION == 1 && TCL_RELEASE_LEVEL != TCL_FINAL_RELEASE))
-#error "Stubs interface doesn't work in 8.0 and alpha/beta 8.1; only 8.1.0+"
-#endif
-
-#ifdef _MSC_VER
-/* Only do this when MSVC++ is compiling us. */
-#ifdef USE_TCL_STUBS
- /* Mark this .obj as needing tcl's Stubs library. */
-#pragma comment(lib, "tclstub" \
-			STRINGIFY(JOIN(TCL_MAJOR_VERSION,TCL_MINOR_VERSION)) ".lib")
-#if !defined(_MT) || !defined(_DLL) || defined(_DEBUG)
-
- /*
-  * This fixes a bug with how the Stubs library was compiled. The
-  * requirement for msvcrt.lib from tclstubXX.lib should be removed.
-  */
-#pragma comment(linker, "-nodefaultlib:msvcrt.lib")
-#endif
-#else
- /* Mark this .obj needing the import library */
-#pragma comment(lib, "tcl" \
-		STRINGIFY(JOIN(TCL_MAJOR_VERSION,TCL_MINOR_VERSION)) ".lib")
-#endif
-#endif
-#undef TCL_STORAGE_CLASS
-#define TCL_STORAGE_CLASS DLLEXPORT
-/* END STUBS MUMBO JUMBO */
+/* Runtime Tcl version, set below */
+double pgtcl_tcl_version;
 
 /*
  * Pgtcl_Init
@@ -64,21 +31,32 @@
 DLLEXPORT int
 Pgtcl_Init(Tcl_Interp *interp)
 {
-	double		tclversion;
 
+	/*
+	 * The version required really should be TCL_VERSION, but this
+	 * can be compiled under 8.5 with stubs and still mostly works
+	 * with a Tcl 8.4 interpreter, so let 8.4 be the minimum version.
+	 */
 #ifdef USE_TCL_STUBS
-	if (Tcl_InitStubs(interp, "8.1", 0) == NULL)
+	if (Tcl_InitStubs(interp, "8.4", 0) == NULL)
 		return TCL_ERROR;
 #endif
 
 	/*
-	 * Tcl versions >= 8.1 use UTF-8 for their internal string
-	 * representation. Therefore PGCLIENTENCODING must be set to UNICODE
-	 * for these versions.
+	 * Get the Tcl version at runtime, which may differ from the compile-time
+	 * version due to use of Tcl stubs. This is used in some commands to
+	 * prevent crashes due to missing stubs functions.
+	 */
+	if (Tcl_GetDouble(interp,
+				Tcl_GetVar(interp, "tcl_version", TCL_GLOBAL_ONLY),
+				&pgtcl_tcl_version) != TCL_OK)
+		pgtcl_tcl_version = 8.4;  /* A reasonable fallback */
+
+	/*
+	 * Note: Removed code to set PGCLIENTENCODING=UNICODE if tcl_version >= 8.1.
+	 * That did not work for Windows because the libpq DLL didn't see the
+	 * environment change. So now this is done when connecting to the database.
 	 */
-	Tcl_GetDouble(interp, Tcl_GetVar(interp, "tcl_version", TCL_GLOBAL_ONLY), &tclversion);
-	if (tclversion >= 8.1)
-		Tcl_PutEnv("PGCLIENTENCODING=UNICODE");
 
 	/* register all pgtcl commands */
 	Tcl_CreateObjCommand(interp,
@@ -195,6 +173,18 @@ Pgtcl_Init(Tcl_Interp *interp)
 						 (ClientData)NULL,
 						 (Tcl_CmdDeleteProc *)NULL);
 
+	Tcl_CreateObjCommand(interp,
+						 "pg_sendquery_prepared",
+						 Pg_sendquery_prepared,
+						 (ClientData)NULL,
+						 (Tcl_CmdDeleteProc *)NULL);
+
+	Tcl_CreateObjCommand(interp,
+						 "pg_sendquery_params",
+						 Pg_sendquery_params,
+						 (ClientData)NULL,
+						 (Tcl_CmdDeleteProc *)NULL);
+
 	Tcl_CreateObjCommand(interp,
 						 "pg_getresult",
 						 Pg_getresult,
@@ -279,6 +269,88 @@ Pgtcl_Init(Tcl_Interp *interp)
 						  (ClientData) NULL,
 						  (Tcl_CmdDeleteProc *) NULL);
 
+	Tcl_CreateObjCommand(interp,
+						  "pg_result_callback",
+						  Pg_result_callback,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+#ifdef HAVE_PQENCRYPTPASSWORD /* PostgreSQL >= 8.2.0 */
+	Tcl_CreateObjCommand(interp,
+						  "pg_encrypt_password",
+						  Pg_encrypt_password,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+#endif
+
+#ifdef HAVE_LO_TRUNCATE /* PostgreSQL >= 8.3.0 */
+	Tcl_CreateObjCommand(interp,
+						  "pg_lo_truncate",
+						  Pg_lo_truncate,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+#endif
+
+#ifdef HAVE_PQDESCRIBEPREPARED /* PostgreSQL >= 8.2.0 */
+	Tcl_CreateObjCommand(interp,
+						  "pg_describe_cursor",
+						  Pg_describe_cursor,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_describe_prepared",
+						  Pg_describe_prepared,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+#endif
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_backend_pid",
+						  Pg_backend_pid,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_server_version",
+						  Pg_server_version,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+#ifdef HAVE_LO_TELL64  /* PostgreSQL >= 9.3.0 */
+	Tcl_CreateObjCommand(interp,
+						  "pg_lo_tell64",
+						  Pg_lo_tell64,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_lo_lseek64",
+						  Pg_lo_lseek64,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_lo_truncate64",
+						  Pg_lo_truncate64,
+						  (ClientData) NULL,
+						  (Tcl_CmdDeleteProc *) NULL);
+#endif
+
+#ifdef HAVE_PQESCAPELITERAL /* PostgreSQL >= 9.0 */
+	Tcl_CreateObjCommand(interp,
+						  "pg_escape_literal",
+						  Pg_escape_l_i,
+						  (ClientData) 1,
+						  (Tcl_CmdDeleteProc *) NULL);
+
+	Tcl_CreateObjCommand(interp,
+						  "pg_escape_identifier",
+						  Pg_escape_l_i,
+						  (ClientData) 2,
+						  (Tcl_CmdDeleteProc *) NULL);
+#endif
+
 	/* Note PACKAGE_VERSION (or VERSION) is provided by the TEA Makefile */
 #ifndef PACKAGE_VERSION
 #ifdef VERSION
